<!--
GENERATED FILE - DO NOT EDIT
This file was generated by [MarkdownSnippets](https://github.com/SimonCropp/MarkdownSnippets).
Source File: /doc/mdsource/Namers.source.md
To change this file edit the source file and then execute ./run_markdown_templates.sh.
-->

<a id="top"></a>

# Namers

<!-- toc -->
## Contents

  * [The Purpose of Namers](#the-purpose-of-namers)
  * [The Parts of Namers](#the-parts-of-namers)
  * [Registering a Custom Namer](#registering-a-custom-namer)
  * [Alternative Namers](#alternative-namers)
    * [SeparateApprovedAndReceivedDirectoriesNamer](#separateapprovedandreceiveddirectoriesnamer)
  * [Approving multiple files from one test](#approving-multiple-files-from-one-test)<!-- endtoc -->

## The Purpose of Namers

`Approvals::verify(text);`

could be written as:

`REQUIRE(text == (loadContentsFromFile("FileName.TestName.approved.txt"));`

Part of Approval Tests' "Convention over Configuration" is to remove this by automatically creating meaningful file names, and therefore it could be written as:

`REQUIRE(text == (loadContentsFromFile(namer.getApprovedFile()));`

"If you **always** have to do something, you should **never** have to do something."

Since all of your `REQUIRE`s would look like this, we can simplify it with the above `Approvals::verify(text);` - and this is enabled by the ApprovalNamers.

## The Parts of Namers

The conventional layout for files saved with `Approvals::verify()` and related functions is:

* `path_to_test_file/FileName.TestName.approved.txt`
* `path_to_test_file/FileName.TestName.received.txt`

The Approval Namer is responsible for creating these two names.

The interface for this is [`ApprovalNamer`](https://github.com/approvals/ApprovalTests.cpp/blob/master/ApprovalTests/core/ApprovalNamer.h).


## Registering a Custom Namer

If you ever want to create a custom namer, Approval Tests has a mechanism to change which namer it uses by default. Please note that you need to create a function that creates new namers.


<!-- snippet: register_default_namer -->
<a id='snippet-register_default_namer'></a>
```cpp
auto default_namer_disposer =
    Approvals::useAsDefaultNamer([]() { return std::make_shared<FakeNamer>(); });
```
<sup><a href='/tests/DocTest_Tests/namers/NamerTests.cpp#L26-L29' title='File snippet `register_default_namer` was extracted from'>snippet source</a> | <a href='#snippet-register_default_namer' title='Navigate to start of snippet `register_default_namer`'>anchor</a></sup>
<!-- endsnippet -->

## Alternative Namers

### SeparateApprovedAndReceivedDirectoriesNamer

The pattern used by this class for file names is:
- `./approved/[test file name].[test name].[extension]`
- `./received/[test file name].[test name].[extension]`

This layout enables Beyond Compare 4 (or any other directory comparison tool) to compare the `approved/` and `received/` directories, and approve one or more files by copying them (without renaming) from `received/` to `approved/`.

The `approved/` and `received/` directories are created automatically.

To register this as your default namer, use:

<!-- snippet: register_separate_directories_namer -->
<a id='snippet-register_separate_directories_namer'></a>
```cpp
auto default_namer_disposer =
    SeparateApprovedAndReceivedDirectoriesNamer::useAsDefaultNamer();
```
<sup><a href='/tests/DocTest_Tests/namers/NamerTests.cpp#L46-L49' title='File snippet `register_separate_directories_namer` was extracted from'>snippet source</a> | <a href='#snippet-register_separate_directories_namer' title='Navigate to start of snippet `register_separate_directories_namer`'>anchor</a></sup>
<!-- endsnippet -->

When using this namer, you will want to add the following line to your `.gitignore` file:

```
**/received/
```

## Approving multiple files from one test 

See [MultipleOutputFilesPerTest](/doc/MultipleOutputFilesPerTest.md#top).

---

[Back to User Guide](/doc/README.md#top)
